@@ -253,7 +253,7 @@ struct bftw_file {
 	/** The length of the file's name. */
 	size_t namelen;
 	/** The file's name. */
-	char name[];
+	char name[]; // _counted_by(namelen + 1)
 };
 
 /**
@@ -1485,7 +1485,8 @@ fail:
 
 /** Check if we should stat() a file asynchronously. */
 static bool bftw_should_ioq_stat(struct bftw_state *state, struct bftw_file *file) {
-	// To avoid surprising users too much, process the roots in order
+	// POSIX wants the root paths to be processed in order
+	// See https://www.austingroupbugs.net/view.php?id=1859
 	if (file->depth == 0) {
 		return false;
 	}
