@@ -21,7 +21,7 @@ struct trie_leaf {
 	/** The length of the key in bytes. */
 	size_t length;
 	/** The key itself, stored inline. */
-	char key[];
+	char key[] _counted_by(length);
 };
 
 /**
