@@ -232,12 +232,28 @@ int bfs_spawn_adddup2(struct bfs_spawn *ctx, int oldfd, int newfd) {
  */
 #define BFS_POSIX_SPAWNP_AFTER_FCHDIR !(__APPLE__ || __NetBSD__)
 
+/**
+ * NetBSD even resolves the executable before file actions with posix_spawn()!
+ */
+#define BFS_POSIX_SPAWN_AFTER_FCHDIR !__NetBSD__
+
 int bfs_spawn_addfchdir(struct bfs_spawn *ctx, int fd) {
 	struct bfs_spawn_action *action = bfs_spawn_action(BFS_SPAWN_FCHDIR);
 	if (!action) {
 		return -1;
 	}
 
+#if __APPLE__
+	// macOS has a bug that causes EBADF when an fchdir() action refers to a
+	// file opened by the file actions
+	for_slist (struct bfs_spawn_action, action, ctx) {
+		if (fd == action->out_fd) {
+			bfs_spawn_clear_posix(ctx);
+			break;
+		}
+	}
+#endif
+
 #if BFS_HAS_POSIX_SPAWN_ADDFCHDIR
 #  define BFS_POSIX_SPAWN_ADDFCHDIR posix_spawn_file_actions_addfchdir
 #elif BFS_HAS_POSIX_SPAWN_ADDFCHDIR_NP
@@ -401,18 +417,40 @@ static bool bfs_resolve_relative(const struct bfs_resolver *res) {
 	return false;
 }
 
+/** Check if the actions include fchdir(). */
+static bool bfs_spawn_will_chdir(const struct bfs_spawn *ctx) {
+	if (ctx) {
+		for_slist (const struct bfs_spawn_action, action, ctx) {
+			if (action->op == BFS_SPAWN_FCHDIR) {
+				return true;
+			}
+		}
+	}
+
+	return false;
+}
+
+/** Check if we can call xfaccessat() before file actions. */
+static bool bfs_can_access_early(const struct bfs_resolver *res, const struct bfs_spawn *ctx) {
+	if (res->exe[0] == '/') {
+		return true;
+	}
+
+	if (bfs_spawn_will_chdir(ctx)) {
+		return false;
+	}
+
+	return true;
+}
+
 /** Check if we can resolve the executable before file actions. */
 static bool bfs_can_resolve_early(const struct bfs_resolver *res, const struct bfs_spawn *ctx) {
 	if (!bfs_resolve_relative(res)) {
 		return true;
 	}
 
-	if (ctx) {
-		for_slist (const struct bfs_spawn_action, action, ctx) {
-			if (action->op == BFS_SPAWN_FCHDIR) {
-				return false;
-			}
-		}
+	if (bfs_spawn_will_chdir(ctx)) {
+		return false;
 	}
 
 	return true;
@@ -442,17 +480,19 @@ static int bfs_resolve_early(struct bfs_resolver *res, const char *exe, const st
 	};
 
 	if (bfs_can_skip_resolve(res, ctx)) {
-		// Do this check eagerly, even though posix_spawn()/execv() also
-		// would, because:
-		//
-		//     - faccessat() is faster than fork()/clone() + execv()
-		//     - posix_spawn() is not guaranteed to report ENOENT
-		if (xfaccessat(AT_FDCWD, exe, X_OK) == 0) {
-			res->done = true;
-			return 0;
-		} else {
-			return -1;
+		if (bfs_can_access_early(res, ctx)) {
+			// Do this check eagerly, even though posix_spawn()/execv() also
+			// would, because:
+			//
+			//     - faccessat() is faster than fork()/clone() + execv()
+			//     - posix_spawn() is not guaranteed to report ENOENT
+			if (xfaccessat(AT_FDCWD, exe, X_OK) != 0) {
+				return -1;
+			}
 		}
+
+		res->done = true;
+		return 0;
 	}
 
 	res->path = getenv("PATH");
@@ -529,6 +569,12 @@ static bool bfs_use_posix_spawn(const struct bfs_resolver *res, const struct bfs
 	}
 #endif
 
+#if !BFS_POSIX_SPAWN_AFTER_FCHDIR
+	if (res->exe[0] != '/' && bfs_spawn_will_chdir(ctx)) {
+		return false;
+	}
+#endif
+
 	return true;
 }
 
